// The "ce" module - imported by os.py
#include "Python.h"
#include "windows.h"


TCHAR *TStringFromStringLen(CHAR *szStr, int cLen) {
	TCHAR *tzStr;

	tzStr=(TCHAR *)LocalAlloc(LPTR, ((cLen+1) * sizeof(TCHAR)));
	wsprintf(tzStr, L"%hs", szStr);
	return tzStr;
}


TCHAR *TStringFromString(CHAR *szStr) {
	return TStringFromStringLen(szStr, strlen(szStr));
}

CHAR *StringFromTStringLen(TCHAR *tzStr, int cLen) {
	TCHAR *tzPtr;
	CHAR  *szPtr;
	CHAR  *szStr;
	int   n;

	szStr=(CHAR *)LocalAlloc(LPTR, ((cLen+1) * sizeof(CHAR)));
	tzPtr=(TCHAR *)tzStr;
	szPtr=szStr;
	for(n=0; n < cLen; n++) {
		*szPtr=(CHAR)*tzPtr;
		szPtr++;
		tzPtr++;
	}
	return szStr;
}

PyObject *PyString_FromTStringAndSize(TCHAR *tzStr, int cLen) {
	PyObject *ob;
	CHAR  *szStr;

	szStr=StringFromTStringLen(tzStr, cLen);
	ob=PyString_FromStringAndSize(szStr, cLen);
	LocalFree(szStr);
	return ob;
}


PyObject *PyString_FromTString(TCHAR *tzStr) {
	return PyString_FromTStringAndSize(tzStr, lstrlen(tzStr));
}


TCHAR *TStringFromPyString(PyObject *ob) {
	return TStringFromStringLen(PyString_AS_STRING((PyStringObject *)ob), PyString_GET_SIZE((PyStringObject *)ob));
}

static PyObject *
os_error()
{
	return PyErr_SetFromErrno(PyExc_OSError);
}

static PyObject *
os_error_with_filename(char *name)
{
	return PyErr_SetFromErrnoWithFilename(PyExc_OSError, name);
}


static PyObject *os_listdir(PyObject *self, PyObject *args)
{	PyObject *obPath, *obList, *obVal;
	WIN32_FIND_DATA fData;
	TCHAR szPath[512]={0};
	TCHAR *lpPath=NULL;
	HANDLE hFile;
	int cLen=0;

	if(!PyArg_ParseTuple(args, "S", &obPath))
		return NULL;
	lpPath=TStringFromPyString(obPath);
	cLen=lstrlen(lpPath);
	if((szPath[cLen-1] != L'/') && (szPath[cLen-1] != L'\\'))
	{	wsprintf(szPath, L"%s\\*.*", lpPath);
	}
	else
	{	wsprintf(szPath, L"%s*.*", lpPath);
	}
	LocalFree(lpPath);

	hFile=FindFirstFile(szPath, &fData);

	if(hFile==INVALID_HANDLE_VALUE)
	{	PyErr_SetString(PyExc_ValueError, "Invalid path");
		return NULL;
	}

	if((obList=PyList_New(0))==NULL)
		return NULL;
	do
	{	if((fData.cFileName[0]==L'.') && ((fData.cFileName[1]==L'\0') ||
		(fData.cFileName[1]==L'.') && (fData.cFileName[2]==L'\0')))
			continue;
		obVal=PyString_FromTString(fData.cFileName);
		if (obVal==NULL)
		{	Py_DECREF(obList);
			obList=NULL;
			break;
		}
		if(PyList_Append(obList, obVal) != 0)
		{	Py_DECREF(obVal);
			Py_DECREF(obList);
			obList=NULL;
			break;
		}
		Py_DECREF(obVal);
	}
	while(FindNextFile(hFile, &fData)==TRUE);
	FindClose(hFile);
	return obList;
}


static PyObject *os_mkdir(PyObject *self, PyObject *args)
{	PyObject *obPath;
	TCHAR *tzPath;
	LONG  lMode;
	BOOL  bResult;

	if(!PyArg_ParseTuple(args, "S|i", &obPath, &lMode))
		return NULL;
	tzPath=TStringFromPyString((PyObject *)obPath);
	Py_BEGIN_ALLOW_THREADS
	bResult=CreateDirectory(tzPath, NULL);
	Py_END_ALLOW_THREADS
	LocalFree(tzPath);
	if (!bResult) return os_error_with_filename(PyString_AsString(obPath));
	Py_INCREF(Py_None);
	return Py_None;
}


static PyObject *os_rename(PyObject *self, PyObject *args)
{	PyObject *obPath1;
	PyObject *obPath2;
	TCHAR *tzPath1;
	TCHAR *tzPath2;
	BOOL  bResult;

	if(!PyArg_ParseTuple(args, "SS", &obPath1, &obPath2))
		return NULL;
	tzPath1=TStringFromPyString((PyObject *)obPath1);
	tzPath2=TStringFromPyString((PyObject *)obPath2);
	Py_BEGIN_ALLOW_THREADS
	bResult=MoveFile(tzPath1, tzPath2);
	Py_END_ALLOW_THREADS
	LocalFree(tzPath1);
	LocalFree(tzPath2);
	if (!bResult) return os_error();
	Py_INCREF(Py_None);
	return Py_None;
}


static PyObject *os_rmdir(PyObject *self, PyObject *args)
{	PyObject *obPath;
	TCHAR *tzPath;
	BOOL  bResult;

	if(!PyArg_ParseTuple(args, "S", &obPath))
		return NULL;
	tzPath=TStringFromPyString((PyObject *)obPath);
	Py_BEGIN_ALLOW_THREADS
	bResult=RemoveDirectory(tzPath);
	Py_END_ALLOW_THREADS
	LocalFree(tzPath);
	if (!bResult) return os_error_with_filename(PyString_AsString(obPath));
	Py_INCREF(Py_None);
	return Py_None;
}


#ifndef	DONT_HAVE_STAT
static PyObject *os_stat(PyObject *self, PyObject *args)
{	CHAR  *path =NULL;
	struct stat st;
	int v;

	if(!PyArg_ParseTuple(args, "s", &path))
		return NULL;
	Py_BEGIN_ALLOW_THREADS
	v=stat(path, &st);
	Py_END_ALLOW_THREADS
	if(v != 0) return os_error_with_filename(path);
	return Py_BuildValue("(llllllllll)", (long)st.st_mode,
					(long)st.st_ino,
					(long)st.st_dev,
					(long)st.st_nlink,
					(long)st.st_uid,
					(long)st.st_gid,
					(long)st.st_size,
					(long)st.st_atime,
					(long)st.st_mtime,
					(long)st.st_ctime);
}
#endif

static PyObject *os_unlink(PyObject *self, PyObject *args)
{	PyObject *obPath;
	TCHAR *tzPath;
	BOOL bResult;

	if(!PyArg_ParseTuple(args, "S", &obPath))
		return NULL;
	tzPath=TStringFromPyString((PyObject *)obPath);
	Py_BEGIN_ALLOW_THREADS
	bResult=DeleteFile(tzPath);
	Py_END_ALLOW_THREADS
	LocalFree(tzPath);
	if (!bResult)
	{	PyErr_SetString(PyExc_OSError, "DeleteFile failed");
		return NULL;
	}
	Py_INCREF(Py_None);
	return Py_None;
}


static PyMethodDef os_methods[]=
{	{"listdir", os_listdir, 1},
	{"mkdir",   os_mkdir, 1},
	{"rename",  os_rename, 1},
	{"rmdir",   os_rmdir, 1},
#ifndef DONT_HAVE_STAT
	{"stat",    os_stat, 1},
#endif
	{"unlink",  os_unlink, 1},
	{NULL, NULL}
};



void initce()
{	PyObject *m, *d;

	m=Py_InitModule("ce", os_methods);
	d=PyModule_GetDict(m);
	PyDict_SetItemString(d, "error", PyExc_OSError);
	PyDict_SetItemString(d, "environ", PyDict_New());
	if(PyErr_Occurred())
		Py_FatalError("Module init failed");
}

