#include <windows.h>
#include <winbase.h>
#include <stdlib.h>
#include <stdarg.h>
#include <tchar.h>
#include "config.h"

#ifndef PYCE_NO_CRT_EMUL

FILE  *stdin=NULL;
FILE  *stdout=NULL;
FILE  *stderr=NULL;

const __int64 FT_EPOCH=116444736000000000;
const __int64 FT_TICKS=10000000;

static struct tm tmLocal;
long _timezone=0;
int  _daylight=0;

int PyOS_InterruptOccurred()
{
	return 0;
}

void PyOS_InitInterrupts()
{
}
void PyOS_FiniInterrupts()
{
}
/* Convert a win32 error code to a C RTL IO error code */
static int Win32ErrorToRTLError(int rc)
{
	return rc;
}


void abort(void)
{	ExitThread((UINT)3);
}


long atol(const char *string)
{	TCHAR ts[64];
	wsprintf(ts, L"%hs", string);
	return wcstol(ts, NULL, 10);
}


double atof(const char *string)
{	TCHAR ts[64];
	wsprintf(ts, L"%hs", string);
	return wcstod(ts, NULL);
}


void *calloc(size_t n, size_t size)
{	return LocalAlloc(LPTR, n * size);
}


int fclose(FILE *f)
{	return CloseHandle(f) ? 0 : Win32ErrorToRTLError(GetLastError());
}


int feof(FILE *f)
{	return SetFilePointer(f, (LONG)0, NULL, FILE_CURRENT) >= GetFileSize(f, NULL);
}


int ferror(FILE *f)
{	DWORD dwError=0;
	dwError=GetLastError();
	if (dwError==ERROR_BROKEN_PIPE || dwError==ERROR_HANDLE_EOF)
		return 1;
	return 0;
}


char * fgets(char *buffer, int n, FILE *f)
{	CHAR *ptr=buffer;
	CHAR *buf=buffer;
	DWORD dw;
	CHAR  ch;

	if(n <= 0) return(NULL);
	T_LOCK(f);
	while(--n)
	{	if(ReadFile(f, &ch, 1, &dw, NULL) && (dw==0))
		{	if(ptr==buffer)
			{	T_UNLOCK(f);
				return(NULL);
			}
			break;
		}
		if ((*ptr++ = ch)=='\n')
			break;
	}
	*ptr='\0';
	T_UNLOCK(f);
	return buf;
}


FILE *fopen(const char *path, const char *mode)
{	TCHAR ts[MAX_PATH+1];
	DWORD dflag=0;
	DWORD cflag=0;
	HANDLE f;
	UINT i;

	for(i=0; i < strlen(mode); i++)
	{	switch(mode[i])
		{	case 'r':
				dflag=GENERIC_READ;
				cflag=OPEN_EXISTING;
				break;
			case 'w':
				dflag=GENERIC_READ|GENERIC_WRITE;
				cflag=CREATE_ALWAYS;
				break;
			case 'a':
				dflag=GENERIC_READ|GENERIC_WRITE;
				cflag=OPEN_ALWAYS;
				break;
			case '+':
				dflag=GENERIC_READ|GENERIC_WRITE;
				break;
			case 'b':
			case 't':
			case 'c':
			case 'n':
				break;
			default:
				return NULL;
		}
	}
	wsprintf(ts, L"%hs", path);
	f=CreateFile(ts, dflag, FILE_SHARE_READ|FILE_SHARE_WRITE,
				 NULL, cflag, FILE_ATTRIBUTE_NORMAL, NULL);
	return f==INVALID_HANDLE_VALUE ? 0 : f;
}


int fputs(const char *str, FILE *f)
{	DWORD dw;
	return WriteFile(f, str, (DWORD)(strlen(str)), &dw, NULL) ? 1 : EOF;
}


size_t fread(void *buffer, size_t size, size_t count, FILE *f)
{	DWORD dw=0; /* Must set to 0 when reading 0 bytes */
	ReadFile(f, buffer, size*count, &dw, NULL);
	return (size_t)(dw/size);
}


int fseek(FILE *f, long offset, int orgin)
{	return (int)SetFilePointer(f, offset, NULL, orgin);
}

long lseek(int handle, long offset, int orgin)
{	return (long)SetFilePointer((HANDLE)handle, offset, NULL, orgin);
}

long ftell(FILE *f)
{	return (long)SetFilePointer(f, 0, NULL, FILE_CURRENT);
}


size_t fwrite(const void *buffer, size_t size, size_t n, FILE *f)
{	DWORD dw;
	WriteFile(f, buffer, (size*n), &dw, NULL);
	return (size_t)(dw/size);
}


int getc(FILE *f)
{	DWORD d=0;
	int   c=0;
	ReadFile((HANDLE)f, (LPVOID)&c, 1, &d, NULL);
	return c;
}


int putc(int c, FILE *f)
{	DWORD dw;
	WriteFile((HANDLE)f, (LPVOID)&c, 1, &dw, NULL);
	return((dw==1) ? c : EOF);
}

int _fileno(FILE *f)
{
	return (int)f;
}

void clearerr(FILE *f)
{
	return;
}

char *strrchr(const char *string, int c)
{	CHAR *p=NULL;
	while(*string)
	{	if(*string==c)
			p=(char*)string;
		string++;
	}
	return p;
}


size_t strspn(const char *string, const char *control)
{	const unsigned char *s=string;
	const unsigned char *c=control;
	unsigned char map[32];
	int i;

	for(i=0; i < 32; i++)
	{	map[i]=0;
	}
	while(*c)
	{	map[*c >> 3] |= (1 << (*c & 7));
		c++;
	}
	if(*s)
	{	i=0;
		while(map[*s >> 3] & (1 << (*s & 7)))
		{	i++;
			s++;
		}
		return(i);
	}
	return(0);
}

/*
void tzset()
{	static int set=0;
	FILETIME ftLocal;
	FILETIME ftUtc;

	if (!set)
	{	memset(&ftLocal, 0, sizeof(ftLocal));
		LocalFileTimeToFileTime(&ftLocal, &ftUtc);
		_timezone=(long)((*(__int64*)&ftUtc)/10000000);
		set=1;
	}
}
*/

int unlink(const char *path)
{	TCHAR wpath[MAX_PATH+1];
	int n;
	n=MultiByteToWideChar(CP_ACP, 0, path, -1, wpath, MAX_PATH);
	wpath[n]=_T('\0');
	n=DeleteFile(wpath) ? 0 : Win32ErrorToRTLError(GetLastError());
	return n;
}


int fprintf(FILE *f, const char* format, ...)
{	va_list args;
	CHAR  buffer[16384];
//32768 65536
	DWORD dw=0;

	memset((void *)buffer, 0, 16384);
	va_start(args, format);
	vsprintf(buffer, format, args);
	va_end(args);
	WriteFile((HANDLE)f, buffer, strlen(buffer), &dw, NULL);
	return (int)dw;
}


int printf(const char* format, ...)
{	va_list args;
	CHAR  buffer[16384];
	DWORD dw;

	memset((void *)buffer, 0, 16384);
	va_start(args, format);
	vsprintf(buffer, format, args);
	va_end(args);
	WriteFile((HANDLE)stdout, buffer, strlen(buffer), &dw, NULL);
	return (int)dw;
}

//#ifndef _WIN32_WCE_EMULATION
int toupper(int c) 	{ return ((c)-'a'+'A'); }
int tolower(int c)  { return ((c)-'A'+'a'); }
int isupper(int c) 	{ return ((c >='A')&&(c<='Z')); }
int islower(int c) 	{ return ((c >='a')&&(c<='z')); }
int isdigit(int c) 	{ return ((c >='0')&&(c<='9')); }
int isxdigit(int c)	{ return (((c >='0')&&(c<='9')) || ((c>='A')&&(c<='F')) || ((c>='a')&&(c<='f'))); }
int isalpha(int c) 	{ return (((c >='A')&&(c<='Z')) || ((c >='a')&&(c<='z'))); }
int isalnum(int c) 	{ return (((c >='A')&&(c<='Z')) || ((c >='a')&&(c<='z')) || ((c >='0')&&(c<='9'))); }
int isspace(int c) 	{ return ((c==' ')||(c=='\t')); }
int isprint(int c)  { return isalnum(c) || isspace(c);}
//#endif




/*
int  argc;
char *argv[];

void crt__init(LPTSTR cmdline)
{
	TCHAR	c ;
	BOOL	f_skip = TRUE, f_quote = FALSE ;

	if ( args ) {
		*args = 0 ;
	}
	argv[ 0 ] = TEXT("") ;
	*argc = 1 ;
	while ( c = *cmdline ) {
		if ( f_skip ) {
			if ( !(c == TEXT(' ') || c == TEXT('\t')) ) {
				if ( c == TEXT('"') ) {
					f_quote = TRUE ;
					cmdline ++ ;
				}
				if ( args && *argc == 2 ) {
					_tcscpy( args, cmdline ) ;
				}
				argv[ (*argc) ++ ] = cmdline ;
				f_skip = FALSE ;
			}
		} else {
			if ( f_quote ) {
				if ( c == TEXT('"') ) {
					f_quote = FALSE ;
					*cmdline = 0 ;
					f_skip = TRUE ;
				}
			} else if ( (c == TEXT(' ') || c == TEXT('\t')) ) {
				*cmdline = 0 ;
				f_skip = TRUE ;
			}
		}
		cmdline ++ ;
	}
}

*/
#endif /* PYCE_NO_CRT_EMUL */

/* Regardless of where fopen/fclose come from, we
   export them as public symbols from Python
*/
FILE *Py_fopen(const char *path, const char *mode)
{
	return fopen(path, mode);
}

int Py_fclose(FILE *f)
{
	return fclose(f);
}
