#ifndef CRTAPI_H
#define CRTAPI_H
#ifdef __cplusplus
extern "C" {
#endif

#ifdef PYCE_NO_CRT_EMUL

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <windows.h>
#include <tchar.h>
//#include <wchar.h>

#else

typedef long                   _off_t;
typedef long                   off_t;

#include <windows.h>

#define EPERM           1
#define ENOENT          2
#define ESRCH           3
#define EINTR           4
#define EIO             5
#define ENXIO           6
#define E2BIG           7
#define ENOEXEC         8
#define EBADF           9
#define ECHILD          10
#define EAGAIN          11
#define ENOMEM          12
#define EACCES          13
#define EFAULT          14
#define EBUSY           16
#define EEXIST          17
#define EXDEV           18
#define ENODEV          19
#define ENOTDIR         20
#define EISDIR          21
#define EINVAL          22
#define ENFILE          23
#define EMFILE          24
#define ENOTTY          25
#define EFBIG           27
#define ENOSPC          28
#define ESPIPE          29
#define EROFS           30
#define EMLINK          31
#define EPIPE           32
#define EDOM            33
#define ERANGE          34
#define EDEADLK         36
/* #define ENAMETOOLONG    38 */
#define ENOLCK          39
#define ENOSYS          40
/* #define ENOTEMPTY       41 */
#define EILSEQ          42

#define _S_IFMT		0170000         /* file type mask */
#define _S_IFDIR	0040000         /* directory */
#define _S_IFCHR	0020000         /* character special */
#define _S_IFIFO	0010000         /* pipe */
#define _S_IFREG	0100000         /* regular */
#define _S_IREAD	0000400         /* read permission, owner */
#define _S_IWRITE	0000200         /* write permission, owner */
#define _S_IEXEC	0000100         /* execute/search permission, owner */

#define S_IFMT		_S_IFMT
#define S_IFDIR		_S_IFDIR
#define S_IFCHR		_S_IFCHR
#define S_IFREG		_S_IFREG
#define S_IREAD		_S_IREAD
#define S_IWRITE	_S_IWRITE
#define S_IEXEC		_S_IEXEC

#define CLOCKS_PER_SEC  1000
#define BUFSIZE         1024
#define BUFSIZ          1024
#undef  SEEK_SET
#define SEEK_SET        FILE_BEGIN
#undef  SEEK_END
#define SEEK_END        FILE_END
#undef	SOOK_CUR
#define SEEK_CUR		FILE_CURRENT
#define EOF             (-1)
#define T_LOCK(x)
#define T_UNLOCK(x)



typedef	void            FILE;
typedef long            clock_t;
typedef long            time_t;
typedef unsigned short 	_ino_t;
typedef unsigned int	_dev_t; 
typedef long			_off_t;



struct stat {
	_dev_t st_dev;
	_ino_t st_ino;
	unsigned short st_mode;
	short st_nlink;
	short st_uid;
	short st_gid;
	_dev_t st_rdev;
	_off_t st_size;
	time_t st_atime;
	time_t st_mtime;
	time_t st_ctime;
	};

struct tm 
{   int tm_sec;
    int tm_min;
    int tm_hour;
    int tm_mday;
    int tm_mon;
    int tm_year;
    int tm_wday;
    int tm_yday;
    int tm_isdst;
};


extern FILE *stdin;
extern FILE *stdout;
extern FILE *stderr;

//#ifndef _MT
//static int errno;
//#endif
static long _timezone;


void abort(void);
long atol(const char *);
double atof(const char *);
void *calloc(size_t, size_t);
void exit(int);
int fclose(FILE *);
int feof(FILE *);
int ferror(FILE *);
char *fgets(char *, int, FILE *);
FILE *fopen(const char *, const char *);
int fprintf(FILE *, const char *, ...);
int fputs(const char *, FILE *);
size_t fread(void *, size_t, size_t, FILE *);
int fseek(FILE *, long, int);
long lseek( int handle, long offset, int origin );
long ftell(FILE *);
size_t fwrite(const void *, size_t, size_t, FILE *);
int getc(FILE *);
int _fileno(FILE *f);
void clearerr(FILE *f);

int printf(const char *, ...);
int putc(int , FILE *);
int sprintf(char *, const char *, ...);
int snprintf(char *, size_t, const char *, ...);
int sscanf(const char *, const char *,...);
int stat(const char *, struct stat *);
int fstat(int handle, struct stat *);
char *strrchr(const char *, int);
size_t strspn(const char *, const char *);
int unlink(const char *);
int vsprintf(char *, const char *, va_list);
int vsnprintf(char *, size_t, const char *, va_list);

int toupper(int);
int tolower(int);
int isupper(int);
int islower(int);
int isdigit(int);
int isxdigit(int);
int isalpha(int);
int isalnum(int);
int isspace(int);





#define _chsize(f, s) -1
#define fflush(f) 0
#define fputc putc


#define getchar() getc(stdin)
#define getenv(v) NULL
#define isatty(f) 0
#define putchar(c) putc(c, stdout)
#define setbuf(f, p)
#define strerror(e) "System Error"

#undef OutputDebugString
#define OutputDebugString printf
#endif /* PYCE_NO_CRT_EMUL */

/* Regardless of where fopen/fclose come from, we
   export them as public symbols from Python
*/
DL_IMPORT(FILE *) Py_fopen(const char *path, const char *mode);
DL_IMPORT(int) Py_fclose(FILE *f);


#ifdef __cplusplus
}
#endif
#endif

