#include "windows.h"
#include "stdlib.h"
#include "limits.h"
#include "config.h"

#ifndef PYCE_NO_CRT_EMUL

#define SIZE_T_MAX 0xffffffffUL


#define is_digit(c) ((c) >= '0' && (c) <= '9')

struct strbuf {
    char *s;
    size_t max, cnt;
};

enum flag {
    flag_none =0,
    flag_minus=1,
    flag_plus =2,
    flag_space=4,
    flag_hash =8,
    flag_zero =16,
    flag_short=32,
    flag_long =64
};

static size_t 
strnlen(const char *s, size_t n)
{
    size_t len = 0;
    while (len < n && *s++ != '\0') { ++len; }
    return len;
}

static void 
put_char(struct strbuf *buf, char c)
{
    if (buf->s != NULL && buf->cnt + 1 < buf->max) {
	buf->s[buf->cnt] = c;
    }
    ++buf->cnt;
}

static void 
put_string(struct strbuf *buf, const char *s, int len, 
		   int width, int prec, enum flag f)
{
    int i;
    if (len < 0) { len = strnlen(s, prec >= 0 ? prec : SIZE_T_MAX); }
    else if (prec >= 0 && prec < len) { len = prec; }
    if (width > len && !(f & flag_minus)) {
	for (i = len; i < width; ++i) { put_char(buf, ' '); }
    }
    for (i = 0; i < len; ++i) { put_char(buf, s[i]); }
    if (width > len && f & flag_minus) {
	for (i = len; i < width; ++i) { put_char(buf, ' '); }
    }
}

static void 
put_ulong(struct strbuf *buf, unsigned long int value, int base, int upper,
		  const char *prefix, int width, int prec, enum flag f)
{
    int len, zeros, i;
    unsigned long int x, x2;
    for (len = 1, x = 1; x < ULONG_MAX / base; ++len, x = x2) {
	x2 = x * base;
	if (x2 > value) { break; }
    }
    zeros = (prec > len) ? prec - len : 0;
    width -= zeros + len;
    if (prefix != NULL) { width -= strnlen(prefix, SIZE_T_MAX); }
    if (!(f & flag_minus)) {
	for (i = 0; i < width; ++i) { put_char(buf, ' '); }
    }
    if (prefix != NULL) { put_string(buf, prefix, -1, 0, -1, flag_none); }
    for (i = 0; i < zeros; ++i) { put_char(buf, '0'); }
    for ( ; x > 0; x /= base) {
	int digit = (value / x) % base;
	put_char(buf, (digit < 10 ? '0' : (upper ? 'A' : 'a') - 10) + digit);
    }
    if (f & flag_minus) {
	for (i = 0; i < width; ++i) { put_char(buf, ' '); }
    }
}

static void 
put_float(struct strbuf *buf, double value, int width, int prec, enum flag f)
{	int c=6, i=0, d=0, n=0;
	char *p=NULL;
	char *v=NULL;

	if (prec > -1) c=prec;
	v=_fcvt(value, c, &d, &n);
	if (n) put_char(buf, '-');
	if (d <= 0) {
		put_char(buf, '0');
		if (c > 0) {
			put_char(buf, '.');
			n=0;
			for (i=d; i < 0; i++) {
				if (n++ < c) put_char(buf, '0');
			}
			p=v;
			for (i=n; i < c; i++) {
				put_char(buf, *p++);
			}
		}
	}
	else {
		p=v;
		for (i=0; i < d; i++) {
			put_char(buf, *p++);
		}
		if (c > 0) {
			put_char(buf, '.');
			for (i=0; i < c; i++) {
				put_char(buf, *p++);
			}
		}
	}
}

static void 
put_double(struct strbuf *buf, double value, int width, int prec, enum flag f)
{	char cb[84]={0};
	int c=6, i=0;
	char *p=NULL;

	if (prec > -1) c=prec;
	p=_gcvt(value, c, cb);

	if (*p == '-')
		put_char(buf, *p++);
	for (; *p != '\0'; p++) {
		if (*p=='.')
			break;
		put_char(buf, *p);
	}
	if (c > 0) {
		p++;
		if (*p != '\0') {
			put_char(buf, '.');
			while (*p != '\0') {
				put_char(buf, *p++);
			}
		}
	}

/*
	if (*p++=='.') {
		if (*p != '\0') {
			put_char(buf, '.');
			for (i=0; i < c; i++) {
				if (*p != '\0') {
					put_char(buf, *p++);
				}
				else {
					put_char(buf, '0');
				}
			}
		}
	}
*/

}

int sprintf(char *s, const char *format, ...)
{	va_list ap;
    int result;
    va_start(ap, format);
	result = vsnprintf(s, SIZE_T_MAX, format, ap);
    va_end(ap);
    return result;
}

int snprintf(char *s, size_t n, const char *format, ...)
{	va_list ap;
    int result;
    va_start(ap, format);
    result = vsnprintf(s, n, format, ap);
    va_end(ap);
    return result;
}

int vsprintf(char *s, const char *format, va_list arg)
{	return vsnprintf(s, SIZE_T_MAX, format, arg);
}

int vsnprintf(char *s, size_t n, const char *format, va_list arg)
{	struct strbuf buf;
    char c;
    buf.s = s;
    buf.max = n;
    buf.cnt = 0;

    while ((c = *format++) != '\0') {

	if (c != '%' || (c = *format++) == '%') {
	    put_char(&buf, c);
	}
	else {
	    enum flag f = flag_none;
	    int width = 0;
	    int prec = -1;

	    for ( ; c != '\0'; c = *format++) {
			if      (c == '-') { f |= flag_minus; }
			else if (c == '+') { f |= flag_plus; }
			else if (c == ' ') { f |= flag_space; }
			else if (c == '#') { f |= flag_hash; }
			else if (c == '0') { f |= flag_zero; }
			else {
				break;
			}
		}

	    if (c == '*') {
			width = va_arg(arg, int);
			if (width < 0) {
			    f |= flag_minus;
		    	width = -width;
			}
			c = *format++;
	    }
		else {
			for ( ; is_digit(c); c = *format++) {
		    	width = width * 10 + (c - '0');
			}
	    }

		if (c == '.') {
			f &= ~flag_zero;
			c = *format++;
			if (c == '*') {
				prec = va_arg(arg, int);
				c = *format++;
			}
			else {
				for (prec = 0; is_digit(c); c = *format++) {
					prec = prec * 10 + (c - '0');
				}
			}
	    }

		if (c == 'h' || c == 'l') {
			f |= (c == 'h' ? flag_short : flag_long);
			c = *format++;
		}

		if (c == 'd' || c == 'i') {
			long int value;
			if (f & flag_short) {
				value = (short int) va_arg(arg, int);
			}
			else if (f & flag_long) {
				value = va_arg(arg, long int);
			}
			else {
				value = va_arg(arg, int);
			}

			if (value >= 0) {
		    	if (f & flag_plus) {
					put_ulong(&buf, value, 10, 0, "+", width, prec, f);
		    	}
				else if (f & flag_space) {
					put_ulong(&buf, value, 10, 0, " ", width, prec, f);
				}
				else {
					put_ulong(&buf, value, 10, 0, NULL, width, prec, f);
				}
			}
			else {
				put_ulong(&buf, -value, 10, 0, "-", width, prec, f);
			}
	    }

		else if (c == 'f') {
			double value = va_arg(arg, double);
			put_float(&buf, value, width, prec, f);
		}

		else if (c == 'g') {
			double value = va_arg(arg, double);
			put_double(&buf, value, width, prec, f);
		}

		else if (c == 'o' || c == 'u' || c == 'x' || c == 'X') {
			unsigned long int value;
			if (f & flag_short) {
				value = (unsigned short int) va_arg(arg, unsigned int);
			}
			else if (f & flag_long) {
				value = va_arg(arg, unsigned long int);
			}
			else {
				value = va_arg(arg, unsigned int);
			}

			if (c == 'o') {
				if (f & flag_hash && value != 0) {
					put_ulong(&buf, value, 8, 0, "0", width, prec, f);
				}
				else {
					put_ulong(&buf, value, 8, 0, NULL, width, prec, f);
				}
			}
			else if (c == 'u') {
				put_ulong(&buf, value, 10, 0, NULL, width, prec, f);
			}
			else {
				if (f & flag_hash && value != 0) {
					if (c == 'x') {
						put_ulong(&buf, value, 16, 0, "0x", width, prec, f);
					}
					else {
						put_ulong(&buf, value, 16, 1, "0X", width, prec, f);
					}
				}
				else {
					put_ulong(&buf, value, 16, 0, NULL, width, prec, f);
				}
			}
	    } 

		else if (c == 'c') {
			char value = va_arg(arg, int);
			put_char(&buf, value);
	    }

		else if (c == 's') {
			const char *value = va_arg(arg, const char *);
			if (value == NULL) {
				put_string(&buf, "(null)", -1, width, prec, f);
			}
			else if (f & flag_hash) {
				put_string(&buf, value + 1, (unsigned char) *value, width, prec, f);
			}
			else {
				put_string(&buf, value, -1, width, prec, f);
			}
		}

		else if (c == 'p') {
			const void *value = va_arg(arg, const void *);
			put_ulong(&buf, (unsigned long int) value, 16, 0, "0x", width, prec, f);
		}

		else if (c == 'n') {
			if (f & flag_short) {
				short int *value = va_arg(arg, short int *);
				*value = buf.cnt;
			}
			else if (f & flag_long) {
				long int *value = va_arg(arg, long int *);
				*value = buf.cnt;
			}
			else {
				int *value = va_arg(arg, int *);
				*value = buf.cnt;
			}
		}

		else {
			put_char(&buf, c);
	    }
	}

	}
	if (s != NULL && n > 0) {
		s[buf.cnt < n ? buf.cnt : n - 1] = '\0';
	}
	return buf.cnt;
}

#endif /* PYCE_NO_CRT_EMUL */
